/******************************************************************************

 odkmchoice - Splits an ODK multiple choice variable into dummies
 
 In order to use this command either of the following two actions needs to 
 be taken:
 
 	1.	Save this do-file with the name odkmchoice.ado (instead of .do)
 		in your PERSONAL ado folder. (Run command -sysdir- in STATA to find 
 		the directory path to your PERSONAL ado folder)
 	   
	2.	Run this do-file as a regular do-file before using odkmchoice. This will 
		only install the command temporarily. There is no issue with installing
		the do-file multiple times, so I recommend executing this do-file in the 
		beginning of any other do-file that uses odkmchoice.


 Description
 
 	This command takes variables containing multiple choice strings from ODK
 	and creates dummy variables labeled yes/no for each category in the multiple
 	choice question. The dummy variables will have the same name as the original
 	variable but with the suffix _dum_1, _dum_2 etc.
 	
 	This command is only suitable when the categories are numbered (1, 2, 3 etc).
 	If the categories are named (a, b, c) or (rice, wheat, maize) this command 
 	will not work. If one integer is skipped when naming the options when 
 	prepareing the survey then add one to the number of categories (option 
 	numcategories). Then drop the dummy that corresponds to the answer option that 
 	was not used after this odkmchoice is executed. For example, if the four 
 	possible answers in a multiple-choice question were numbered (1, 2, 3, 5) 
 	there are four possible categories but the highest number in the string will
 	be five. A solution to this is to set the numcategories option to five and 
 	after executing odkmchoice drop the dummy for the answer option that was not
 	impossible, in this case the dummy with suffix _dum_4.
 	
 	odkmchoice will treat any error code (-8, -999 etc) as missing values and all
 	dummies will have missing values. If an correct answer is combined with an
 	error value, for example "1 2 -9" then _dum_1 and _dum_2 will be "yes", and 
 	all other values will be "no", but no missing values for that observation.
 
 
 Syntax
 
 	odkmchoice varname, numcategories(integer) [drop]
 
 	varname:		Only use this command on one variable at the time.
 	
 	numcategories:	This option is required. The number of categories used for
 					this multiple choice question. Shortest abbriviation is numc()
 					
 	drop:			Optional option to drop original variable after dummies 
 					are created 
 	
 					
 Examples
 
 	
 	Respondents answered which out of 8 crops they grew. Keeping original 
 	variable.Creating dummy variables named crops_grown_dum_1, 
 	crops_grown_dum_2 ... crops_grown_dum_8.
 	
 		odkmchoice crops_grown, numcategories(8)
 	
 		
 	Respondents answered which out of 4 types of savings they had, dropping the
 	original variable.
 	
 		odkmchoice types_of_savings, numc(4) drop
 
 					

******************************************************************************/


capture program drop odkmchoice
program odkmchoice
	qui {	
		syntax varname, NUMCategories(integer) [drop]
			
		split `varlist', generate(`varlist'_split_)
		local split_varNo `r(nvars)'
		
		forvalues dumNo = 1/`numcategories' {
			*Generating dummy varaibles						
			gen byte `varlist'_dum_`dumNo'	= 0
			
			forvalues split_var = 1/`split_varNo' {
				*Searching for variable corresponding to each dummy variable in the split variables
				replace `varlist'_dum_`dumNo'	= 1		if `varlist'_split_`split_var' == "`dumNo'"
			}
			
			** Observations with missing multiple choice should not have "no" in the dummy, it should be missing.
			replace `varlist'_dum_`dumNo'	= .		if `varlist' == ""
			
			
			*Adding yes no labels to dummy variables
			label define dum_yes_no_code 0 "No" 1 "Yes", modify 
			label values `varlist'_dum_`dumNo' dum_yes_no_code	
			
			sum `varlist'_dum_`dumNo'
			if `r(sum)' == 0 {
				noi di `"`varlist'_dum_`dumNo': All observations "no" or missing"'
			}
		}

		drop `varlist'_split_*
		
		/** Replacing dummy values to missing if the string variable is not empt but do not have a valid answer, for example error codes etc.
		egen rowcount 	= rowtotal(`varlist'_dum_*)
		forvalues dumNo = 1/`numcategories' {		
			replace `varlist'_dum_`dumNo' = .	if rowcount == 0
		}	
		drop rowcount*/
		
		* Other options 
		gen 	`varlist'_dum_dk=(`varlist'=="-88")
		replace `varlist'_dum_dk=. if `varlist'==""
		gen 	`varlist'_dum_refuse=(`varlist'=="-66")
		replace `varlist'_dum_refuse=. if `varlist'==""
		gen 	`varlist'_dum_na=(`varlist'=="-77")
		replace `varlist'_dum_na=. if `varlist'==""
		gen 	`varlist'_dum_other=(`varlist'=="99" | `varlist'=="other")
		replace `varlist'_dum_other=. if `varlist'==""
		
		lab val `varlist'_dum_dk     dum_yes_no_code
		lab val `varlist'_dum_refuse dum_yes_no_code
		lab val `varlist'_dum_na     dum_yes_no_code
		lab val `varlist'_dum_other  dum_yes_no_code

		
		
		* Replace by missing all dummys if original was missing 
		forvalues dumNo = 1/`numcategories' {		
			replace `varlist'_dum_`dumNo'=.  if `varlist'=="."
		}	
		*Moving the dummy varaibles so that it comes after the original varaible
		order `varlist'_dum_*, after(`varlist')
		
		
		*If the option drop is included, the original variable is dropped.
		if "`drop'" == "drop" {
			*Droping the original variable
			drop `varlist'
		}
	}
end
 