#------------------------------------------------------------------------------#
#                                                                              #
# Cash Is Queen                                                                #
# Local Economy Effects of Cash Transfers to Women in West Africa              #
# Replication File                                                             #		
# Appendix B: Further Details on Spatial Modeling of Spillovers  			         #                  
#------------------------------------------------------------------------------#

#  PURPOSE: This R script was written to produce graphs for the paper "Cash is Queen"
#  WRITTEN BY: Paula Gonzalez
#  Last modified in: March 2025
## *************************************************************************** ##

#### PART 1: MAIN SETTINGS  ####################################################

### 1.1) Clear Environment --------------------------------------------------
rm(list = ls())  # Remove all objects

### 1.2) Load Required Packages ----------------------------------------------

# Define required packages
required_packages <- c("ggplot2", "foreign", "ggridges", "dplyr", "plyr", "rio", "tidyverse", "viridis")

# Install missing packages
new_packages <- required_packages[!(required_packages %in% installed.packages()[, "Package"])]
#if (length(new_packages)) install.packages(new_packages, dependencies = TRUE)
if (length(new_packages)) install.packages(new_packages)

# Load packages
suppressMessages(lapply(required_packages, library, character.only = TRUE))

# Set default theme for ggplot
theme_set(theme_minimal())

### 1.3) Set Folder Paths ----------------------------------------------------

# Define root folder based on username
projectFolder <- "C:/WBG/rep-checks/RR_NGA_2025_291"

# Define subfolders
Data        <- file.path(projectFolder, "Data")
Output      <- file.path(projectFolder, "Outputs")
rawData     <- file.path(Data, "Raw")
intData     <- file.path(Data, "Intermediate")
finalData   <- file.path(Data, "Final")
rawOutput   <- file.path(Output, "Raw")
finalOutput <- file.path(Output, "Final")

rawFigure   <- file.path(rawOutput, "Figures")
finalFigures   <- file.path(finalOutput, "Figures")



# Ensure output directories exist
dir.create(Output, recursive = TRUE, showWarnings = FALSE)
dir.create(finalOutput, recursive = TRUE, showWarnings = FALSE)

### 1.4) Import Dataset from Stata --------------------------------------------

# Import data safely
data_path <- file.path(rawData, "HHSpatialLongData.dta")

if (!file.exists(data_path)) {
  stop(paste("Error: Dataset file not found at:", data_path))
}

data_dist <- rio::import(data_path)

# Subset treatment and non-treatment groups
data_treat   <- subset(data_dist, village_ineligible_ct == 0)
data_notreat <- subset(data_dist, village_ineligible_ct == 1)

#### PART 2: GRAPHS ###########################################################
windowsFonts(A = windowsFont("Times New Roman")) 

### 2.1) Graph: Proportion of Cash Transfers (PET) in Treatment Villages -----
ggplot(
  data_treat, 
  aes(x =pet, as.factor(meters), fill = stat(x))) +
  geom_density_ridges_gradient(scale = 3, size = 0.3, rel_min_height = 0.01) +
  scale_fill_viridis_c(name = "Fraction", option = "C") +
  scale_x_continuous(name="Cash Transfer Recipients Neighbors(%)", limits=c(0, 1))+
  labs(title = 'Proportion of Cash Transfers (PET)',
       subtitle= 'Cash Transfers Treatment Villages',
       x ="Fraction", y = "Local neighborhood radius in meters") +
  theme(text = element_text(family = "A"))

# Save the last displayed plot
ggsave(file.path(finalFigures, "FB5_PET_CTVillages.png"), width = 8, height = 6, dpi = 300)



### 2.2) Graph: Proportion of Extreme Vulnerable Neighbors (PEV) -----
ggplot(
  data_dist, 
  aes(x =pEV, y=as.factor(meters), fill = stat(x))) +
  geom_density_ridges_gradient(scale = 3, size = 0.3, rel_min_height = 0.01) +
  scale_fill_viridis_c(name = "Fraction", option = "C") +
  scale_x_continuous(name="Extreme Vulnerable Neighbors(%)", limits=c(0, 1))+
  labs(title = 'Proportion of Eligibles (PEV)',
       subtitle= 'All Villages',y = "Local neighborhood radius in meters") +
  theme(text = element_text(family = "A"))

# Save the last displayed plot
ggsave(file.path(finalFigures, "FB5_PEV_ALLVillages.png"), width = 8, height = 6, dpi = 300)


### 2.3) Graph: Market Size (#HH) for All Villages -----
ggplot(
  data_dist, 
  aes(x =bpov, as.factor(meters), fill = stat(x))) +
  geom_density_ridges_gradient(scale = 3, size = 0.3, rel_min_height = 0.01) +
  scale_fill_viridis_c(name = "Frequency", option = "C") +
  scale_x_continuous(name="Number of Households", limits=c(-0.1, 100))+
  labs(title = 'Market Size (#HH)',
       subtitle= 'All Villages',
       y = "Local neighborhood radius in meters") +
  theme(text = element_text(family = "A"))

# Save the last displayed plot
ggsave(file.path(finalFigures, "FB5_HH_ALLVillages.png"), width = 8, height = 6, dpi = 300)


#------------------------------------------------------------------------------#
# END OF SCRIPT                                                                #
#------------------------------------------------------------------------------#

