#------------------------------------------------------------------------------#
#                                                                              #
# Cash Is Queen                                                                #
# Local Economy Effects of Cash Transfers to Women in West Africa              #
# Replication File                                                             #		
# Appendix D: Regression Discontinuity Design (RDD) Details 			             #                  
#------------------------------------------------------------------------------#

#  PURPOSE: This R script was written to produce graphs for the paper "Cash is Queen"
#  WRITTEN BY: Paula Gonzalez
#  Last modified in: March 2025
## *************************************************************************** ##

#### PART 1: MAIN SETTINGS  ####################################################

### 1.1) Clear Environment --------------------------------------------------
rm(list = ls())  # Remove all objects

### 1.2) Load Required Packages ----------------------------------------------
### 1.2) PACKAGES   -----------------------------------------------------
# if installing individual packages. 
# install.packages("cobalt")
#install.packages("WeightIt")
#install.packages("jtools")
#install.packages("MatchIt")
#install.packages("mlogit")

# Load of packages
library(cobalt)
library(WeightIt)
library(jtools)
library(ggplot2)
library(MatchIt)
library(mlogit)



### 1.3) Set Folder Paths ----------------------------------------------------
# Define root folder based on username
projectFolder <- "C:/WBG/rep-checks/RR_NGA_2025_291"

# Define subfolders
Data        <- file.path(projectFolder, "Data")
Output      <- file.path(projectFolder, "Outputs")
rawData     <- file.path(Data, "Raw")
intData     <- file.path(Data, "Intermediate")
finalData   <- file.path(Data, "Final")
rawOutput   <- file.path(Output, "Raw")
finalOutput <- file.path(Output, "Final")

rawFigure   <- file.path(rawOutput, "Figures")
finalFigures   <- file.path(finalOutput, "Figures")


### 1.1) Import Dataset from Stata --------------------------------------------
# Importing raw dataset sfrom stata 
data_analysis <- rio::import(file.path(finalData,"FNLPDataWideAnalysis.dta"))


# Subsetting Dataset we need 
# Selecting variables we need
myvars <- c("hhid",
            "age_hhhead_bl",  	   
            "marital_status_bl",     
            "literacy_hhhead_bl",    
            "num_hh_members_bl",   
            "hhtype_bl",    
            "w_daily_adulteq_expenses_bl", 
            "num_plots_bl", 
            "business_work_bl_w", 
            "empowered_new_bl",
            "weai_new_bl_w",
            "lga_new_bl",  
            "d_lgacode_bl_1",
            "CTvill",
            "spill_treatment",
            "spill_treat_cat",
            "d_ct_nct_inel_1", 
            "d_ct_nct_inel_2",
            "d_ct_nct_inel_3", 
            "RDD_18")

# Keeping data just for EV and treatment observations Full sample and RDD
data_graph <- subset(data_analysis,d_vuln_cat_1==1 & FNLP_treatment==0)
data_graph_RDD18 <- subset(data_graph,RDD_18==1) 


### 1.2) Computing Weights   -------------------------------------
# Lopez and Gutman (2017) - maximum pairwise standardized bia
# https://projecteuclid.org/journals/statistical-science/volume-32/issue-3/Estimation-of-Causal-Effects-with-Multiple-Treatments--A-Review/10.1214/17-STS612.full  
#Using WeightIt to generate weights with multinomial
#No logistic /covariate balancing propensity score weighting 
W.out.mn <- WeightIt::weightit(
  spill_treat_cat ~ age_hhhead_bl + marital_status_bl +
    literacy_hhhead_bl + num_hh_members_bl+
    num_plots_bl+weai_new_bl_w+d_lgacode_bl_1,
  data = data_graph_RDD18, method = "ps", use.mlogit = FALSE)


### 1.3) Love Plot multi-category treatments  -------------------------------------
##We want to display standardized mean difference for all of the covariates
set.cobalt.options(binary = "std")

## Options to add 
#Name of variables 
new.names <- c(age_hhhead_bl = "Age (Years)",
               marital_status_bl = "Marital Status (Married=1)",
               literacy_hhhead_bl = "Literacy (Yes=1)",
               num_hh_members_bl = "Number of household members",
               num_plots_bl = "Number of agriculture plots (0-7)",
               weai_new_bl_w = "Empowered WEIA(0/1)",
               d_lgacode_bl_1 = "Birnin Kebbi")
#Setting up the fount
windowsFonts(A = windowsFont("Times New Roman")) 


# Plotting Results - Standardized Difference across groups
love.plot(W.out.mn, 
          binary = "std",
          line = TRUE,
          which.treat = .all, 
          abs = FALSE,
          var.names = new.names,
          colors = c("purple3", "goldenrod2"),
          shapes = c("circle filled", "circle filled"),
          sample.names = c("Unweighted", "PS Weighted"),
          position = "top",
          wrap = 20,
          stars = "raw",
          thresholds = c(m = .1))+
  theme(legend.position = "top",
        legend.box.background = element_rect(), 
        legend.box.margin = margin(1, 1, 1, 1),
        text = element_text(family = "A"))


# Save the last displayed plot
ggsave(file.path(finalFigures, "FD2_CovariateBalance.pdf"), device=NULL, width = 10, 
       height = 8, units =c("cm"))




#------------------------------------------------------------------------------#
# END OF SCRIPT                                                                #
#------------------------------------------------------------------------------#




